@@ -8,12 +8,8 @@ DBT_INT1D     =  4
 DBT_DOUBLE1D  =  5
 DBT_COMPLEX1D =  6
 DBT_STRING1D  =  7
-DBT_INT2D     =  8
-DBT_DOUBLE2D  =  9
-DBT_COMPLEX2D = 10
-DBT_STRING2D  = 11
-DBT_BOOL      = 12
-DBT_INTND     = 13
-DBT_DOUBLEND  = 14
-DBT_COMPELXND = 15
-DBT_UNKNOWN   = 16
+DBT_BOOL      =  8
+DBT_INTND     =  9
+DBT_DOUBLEND  = 10
+DBT_COMPELXND = 11
+DBT_UNKNOWN   = 12
