@@ -20,10 +20,6 @@ typedef enum
   DBT_DOUBLE1D,
   DBT_COMPLEX1D,
   DBT_STRING1D,
-  DBT_INT2D,
-  DBT_DOUBLE2D,
-  DBT_COMPLEX2D,
-  DBT_STRING2D,
   DBT_BOOL,
   DBT_INTND,
   DBT_DOUBLEND,
