@@ -30,11 +30,16 @@ class GaussianLikelihood(object):
     def __init__(self, options):
         self.options=options
         self.data_x, self.data_y = self.build_data()
+        self.likelihood_only = options.get_bool('likelihood_only', False)
 
         if self.constant_covariance:
             self.cov = self.build_covariance()
             self.inv_cov = self.build_inverse_covariance()
 
+            if not self.likelihood_only:
+                self.chol = np.linalg.cholesky(self.cov)
+
+
             # We may want to include the normalization of the likelihood
             # via the log |C| term.
             include_norm = self.options.get_bool("include_norm", False)
@@ -61,8 +66,6 @@ class GaussianLikelihood(object):
         if options.has_value("like_name"):
             self.like_name = options['like_name']
 
-        self.likelihood_only = options.get_bool('likelihood_only', 'False')
-
 
 
     def build_data(self):
@@ -212,6 +215,10 @@ class GaussianLikelihood(object):
             #Also save a simulation of the data - the mean with added noise
             #these can be used among other places by the ABC sampler.
             #This also requires the cov mat.
+            # If we have a parameter-dependent covariance we need
+            # to re-calculate the Cholesky decomposition to simulate some data.
+            if not self.constant_covariance:
+                self.chol = np.linalg.cholesky(self.cov)
             sim = self.simulate_data_vector(x)
             block[names.data_vector, self.like_name + "_simulation"] = sim
 
@@ -220,7 +227,7 @@ class GaussianLikelihood(object):
         #generate a vector of normal deviates
 
         r = np.random.randn(x.size)
-        return x + np.dot(self.cov, r)
+        return x + np.dot(self.chol, r)
 
 
     def extract_theory_points(self, block):
@@ -282,7 +289,7 @@ class SingleValueGaussianLikelihood(GaussianLikelihood):
                 file".format(self.like_name))
         if options.has_value("like_name"):
             self.like_name = options["like_name"]
-        print('Likelihood "{0}" will be Gaussian {1} +/- {2} '.format(self.like_name, self.mean, self.sigma))
+        print('Likelihood "{0}" will be Gaussian {1} +/- {2} '.format(self.like_name, mean, sigma))
         self.data_y = np.array([mean])
         self.cov = np.array([[sigma**2]])
         self.inv_cov = np.array([[sigma**-2]])
