@@ -30,16 +30,6 @@ from . import output as output_module
 RUNTIME_INI_SECTION = "runtime"
 
 
-# TODO: find better home for this.  Utils?
-class ParseExtraParameters(argparse.Action):
-    def __call__(self, parser, args, values, option_string=None):
-        result = {}
-        for arg in values:
-            section, param_value = arg.split('.',1)
-            param,value = param_value.split('=',1)
-            result[(section,param)] = value
-        setattr(args, self.dest, result)
-
 def experimental_fault_handling():
     import  cosmosis.runtime.handler
     cosmosis.runtime.handler.activate_experimental_fault_handling()
@@ -101,33 +91,137 @@ def demo_20b_special (args):
         print ("********************************************************")
         print ("*** YOU MUST RUN demo20a BEFORE YOU CAN RUN demo20b. ***")
         print ("********************************************************")
-        
+
+def sampler_main_loop(sampler, output, pool):
+    # Run the sampler until convergence
+    # which really means "finished" here - 
+    # a sampler can "converge" just by reaching the 
+    # limit of the number of samples it is allowed.
+    if not pool or pool.is_master():
+        while not sampler.is_converged():
+            sampler.execute()
+            #Flush any output. This is to stop
+            #a problem in some MPI cases where loads
+            #of output is built up before being written.
+            if output:
+                output.flush()
+        # If we are in parallel tell the other processors to end the 
+        # loop and prepare for the next sampler
+        if pool and sampler.is_parallel_sampler:
+            pool.close()
+    else:
+        if sampler.is_parallel_sampler:
+            sampler.worker()
+
+
+
+def write_header_output(output, params, values, pipeline):
+    # If there is an output file, save the ini information to
+    # it as well.  We do it here since it's nicer to have it
+    # after the sampler options that get written in sampler.config.
+    # Create a buffer to store the output:
+    output.comment("START_OF_PARAMS_INI")
+    comment_wrapper = output.comment_file_wrapper()
+    params.write(comment_wrapper)
+    output.comment("END_OF_PARAMS_INI")
+    # Do the same with the values file.
+    # Unfortunately that means reading it in again;
+    # if we ever refactor this bit we could eliminate that.
+    if values is None:
+        values_ini=Inifile(pipeline.values_filename)
+    else:
+        values_ini=values
+    output.comment("START_OF_VALUES_INI")
+    values_ini.write(comment_wrapper)
+    output.comment("END_OF_VALUES_INI")
+
+    # And the same with the priors
+    output.comment("START_OF_PRIORS_INI")
+    for priors_file in pipeline.priors_files:
+        prior_ini=Inifile(priors_file)
+        prior_ini.write(comment_wrapper)
+    output.comment("END_OF_PRIORS_INI")
+
+def setup_output(sampler_class, sampler_number, ini, pool, number_samplers, sample_method):
+
+    needs_output = sampler_class.needs_output and \
+       (pool is None or pool.is_master() or sampler_class.parallel_output)
+
+    if not needs_output:
+        return None
+
+
+    #create the output files and methods.
+    try:
+        output_options = dict(ini.items('output'))
+    except configparser.NoSectionError:
+        raise ValueError("ERROR:\nFor the sampler (%s) you chose in the [runtime] section of the ini file I also need an [output] section describing how to save results\n\n"%sample_method)
+    #Additionally we tell the output here if
+    #we are parallel or not.
+    if (pool is not None) and (sampler_class.parallel_output):
+        output_options['rank'] = pool.rank
+        output_options['parallel'] = pool.size
+
+    #Give different output filenames to the different sampling steps
+    #Only change if this is not the last sampling step - the final
+    #one retains the name in the output file.
+    # Change, e.g. demo17.txt to demo17.fisher.txt
+    if ("filename" in output_options) and (sampler_number<number_samplers-1):
+        filename = output_options['filename']
+        filename, ext = os.path.splitext(filename)
+        filename += '.' + sampler_name
+        filename += ext
+        output_options['filename'] = filename
+
+
+    #Generate the output from a factory
+    output = output_module.output_from_options(output_options)
+    output.metadata("sampler", sample_method)
+
+    if ("filename" in output_options):
+        print("* Saving output -> {}".format(output_options['filename']))
+
+    return output
+
 
 def run_cosmosis(args, pool=None, ini=None, pipeline=None, values=None):
     # In case we need to hand-hold a naive demo-10 user.
-    demo_10_special (args)
-    demo_20b_special (args)
 
     # Load configuration.
     if ini is None:
         ini = Inifile(args.inifile, override=args.params)
 
+    pre_script = ini.get(RUNTIME_INI_SECTION, "pre_script", fallback="")
+    post_script = ini.get(RUNTIME_INI_SECTION, "post_script", fallback="")
 
+    if (pool is None) or pool.is_master():
+        # This decodes the exist status
+        status = os.WEXITSTATUS(os.system(pre_script))
+        if status:
+            raise RuntimeError("The pre-run script {} retuned non-zero status {}".format(
+                pre_script, status))
 
     # Create pipeline.
     if pipeline is None:
+        cleanup_pipeline = True
         pool_stdout = ini.getboolean(RUNTIME_INI_SECTION, "pool_stdout", fallback=False)
         if (pool is None) or pool.is_master() or pool_stdout:
-            pipeline = LikelihoodPipeline(ini, override=args.variables, values=values)
-            if pipeline.do_fast_slow:
-                pipeline.setup_fast_subspaces()
-
+            pipeline = LikelihoodPipeline(ini, override=args.variables, values=values, only=args.only)
         else:
             # Suppress output on everything except the master process
-            with stdout_redirected():
-                pipeline = LikelihoodPipeline(ini, override=args.variables, values=values)
-                if pipeline.do_fast_slow:
-                    pipeline.setup_fast_subspaces()
+            if pool_stdout:
+                pipeline = LikelihoodPipeline(ini, override=args.variables, only=args.only) 
+            else:
+                with stdout_redirected():
+                    pipeline = LikelihoodPipeline(ini, override=args.variables, only=args.only) 
+
+        if pipeline.do_fast_slow:
+            pipeline.setup_fast_subspaces()
+    else:
+        # We should not cleanup a pipeline which we didn't make
+        cleanup_pipeline = False
+
+
 
 
     # determine the type(s) of sampling we want.
@@ -165,55 +259,20 @@ def run_cosmosis(args, pool=None, ini=None, pipeline=None, values=None):
             zip(sampler_classes, sample_methods)):
         sampler_name = sampler_class.__name__[:-len("Sampler")].lower()
 
+        # The resume feature lets us restart from an existing file.
+        # It's not fully rolled out to all the suitable samplers yet though.
+        resume = ini.getboolean(RUNTIME_INI_SECTION, "resume", fallback=False)
+
+        # Not all samplers can be resumed.
+        if resume and not sampler_class.supports_resume:
+            print("NOTE: You set resume=T in the [runtime] section but the sampler {} does not support resuming yet.  I will ignore this option.".format(sampler_name))
+            resume=False
+
         if pool is None or pool.is_master():
             print("****************************")
             print("* Running sampler {}/{}: {}".format(sampler_number+1,number_samplers, sampler_name))
 
-
-        if sampler_class.needs_output and \
-           (pool is None 
-            or pool.is_master() 
-            or sampler_class.parallel_output):
-
-            #create the output files and methods.
-            try:
-                output_options = dict(ini.items('output'))
-            except configparser.NoSectionError:
-                sys.stderr.write("ERROR:\nFor the sampler (%s) you chose in the [runtime] section of the ini file I also need an [output] section describing how to save results\n\n"%sample_method)
-                sys.exit(1)
-            #Additionally we tell the output here if
-            #we are parallel or not.
-            if (pool is not None) and (sampler_class.parallel_output):
-                output_options['rank'] = pool.rank
-                output_options['parallel'] = pool.size
-
-            #Give different output filenames to the different sampling steps
-            #Only change if this is not the last sampling step - the final
-            #one retains the name in the output file.
-            # Change, e.g. demo17.txt to demo17.fisher.txt
-            if ("filename" in output_options) and (sampler_number<number_samplers-1):
-                filename = output_options['filename']
-                filename, ext = os.path.splitext(filename)
-                filename += '.' + sampler_name
-                filename += ext
-                output_options['filename'] = filename
-
-
-            #Generate the output from a factory
-            output = output_module.output_from_options(output_options)
-            output.metadata("sampler", sample_method)
-
-            if ("filename" in output_options):
-                print("* Saving output -> {}".format(output_options['filename']))
-
-
-
-        else:
-            #some samplers, like the test one, do not need an output
-            #file of the usual type.  In fact giving them one would be
-            #a bad idea, because they might over-write something important.
-            #so we just give them none.
-            output = None
+        output = setup_output(sampler_class, sampler_number, ini, pool, number_samplers, sample_method)
         print("****************************")
 
         #Initialize our sampler, with the class we got above.
@@ -230,64 +289,40 @@ def run_cosmosis(args, pool=None, ini=None, pipeline=None, values=None):
         sampler.distribution_hints.update(distribution_hints)
         sampler.config()
 
-        #If there is an output file, save the ini information to
-        #it as well.  We do it here since it's nicer to have it
-        #after the sampler options that get written in sampler.config.
-        if output is not None:
-            #Create a buffer to store the output:
-            output.comment("START_OF_PARAMS_INI")
-            comment_wrapper = output.comment_file_wrapper()
-            ini.write(comment_wrapper)
-            output.comment("END_OF_PARAMS_INI")
-            #Do the same with the values file.
-            #Unfortunately that means reading it in again;
-            #if we ever refactor this bit we could eliminate that.
-            if values is None:
-                values_ini=Inifile(pipeline.values_filename)
-            else:
-                values_ini=values
-            output.comment("START_OF_VALUES_INI")
-            values_ini.write(comment_wrapper)
-            output.comment("END_OF_VALUES_INI")
+        # Potentially resume
+        if resume and sampler_class.needs_output and \
+            sampler_class.supports_resume and \
+           (pool is None 
+            or pool.is_master() 
+            or sampler_class.parallel_output):
+           sampler.resume()
 
-            output.comment("START_OF_PRIORS_INI")
-            for priors_file in pipeline.priors_files:
-                prior_ini=Inifile(priors_file)
-                prior_ini.write(comment_wrapper)
-            output.comment("END_OF_PRIORS_INI")
-
-
-        # Run the sampler until convergence
-        # which really means "finished" here - 
-        # a sampler can "converge" just by reaching the 
-        # limit of the number of samples it is allowed.
-        if not pool or pool.is_master():
-            while not sampler.is_converged():
-                sampler.execute()
-                #Flush any output. This is to stop
-                #a problem in some MPI cases where loads
-                #of output is built up before being written.
-                if output:
-                    output.flush()
-            # If we are in parallel tell the other processors to end the 
-            # loop and prepare for the next sampler
-            if pool and sampler.is_parallel_sampler:
-                pool.close()
-        else:
-            if sampler.is_parallel_sampler:
-                sampler.worker()
+        if output:
+            write_header_output(output, ini, values, pipeline)
 
+        sampler_main_loop(sampler, output, pool)
 
         distribution_hints.update(sampler.distribution_hints)
 
         if output:
             output.close()
 
-    pipeline.cleanup()
+    if cleanup_pipeline:
+        pipeline.cleanup()
 
-    # Extra-special actions we take to mollycoddle a brand-new user!
-    demo_1_special (args)
-    demo_20a_special (args)
+
+    # User can specify in the runtime section a post-run script to launch.
+    # In general this may be less useful than the pre-run script, because
+    # often chains time-out instead of actually completing.
+    # But we still offer it
+    if (pool is None) or pool.is_master():
+        # This decodes the exist status
+        status = os.WEXITSTATUS(os.system(post_script))
+        if status:
+            sys.stdout.write("WARNING: The post-run script {} failed with error {}".format(
+                post_script, error))
+
+    return 0
 
 
 def main():
@@ -300,21 +335,26 @@ def main():
         parser.add_argument("--experimental-fault-handling",action='store_true',help="Activate an experimental fault handling mode.")
         parser.add_argument("-p", "--params", nargs="*", action=ParseExtraParameters, help="Override parameters in inifile, with format section.name1=value1 section.name2=value2...")
         parser.add_argument("-v", "--variables", nargs="*", action=ParseExtraParameters, help="Override variables in values file, with format section.name1=value1 section.name2=value2...")
+        parser.add_argument("--only", nargs="*", help="Fix all parameters except the ones listed")
         args = parser.parse_args(sys.argv[1:])
 
+
+        demo_10_special (args)
+        demo_20b_special (args)
+
         if args.experimental_fault_handling:
             experimental_fault_handling()
 
         # initialize parallel workers
         if args.mpi:
             with mpi_pool.MPIPool() as pool:
-                run_cosmosis(args,pool)
+                return run_cosmosis(args,pool)
         elif args.smp:
             with process_pool.Pool(args.smp) as pool:
-                run_cosmosis(args,pool)
+                return run_cosmosis(args,pool)
         else:
             try:
-                run_cosmosis(args)
+                return run_cosmosis(args)
             except Exception as error:
                 if args.pdb:
                     print("There was an exception - starting python debugger because you ran with --pdb")
@@ -324,8 +364,13 @@ def main():
                     raise
     except CosmosisConfigurationError as e:
         print(e)
-        sys.exit (1)
+        return 1
+
+    # Extra-special actions we take to mollycoddle a brand-new user!
+    demo_1_special (args)
+    demo_20a_special (args)
 
 
 if __name__=="__main__":
-    main()
+    status = main()
+    sys.exit(status)
