@@ -42,10 +42,11 @@ class FitsOutput(OutputBase):
             filename = filename[:-len(self.FILE_EXTENSION)]
 
         if nchain > 1:
-            self._filename = "%s_%d%s" % (filename, rank+1, 
-                                          self.FILE_EXTENSION)
-        else:
-            self._filename = filename + self.FILE_EXTENSION
+            filename = filename + "_{}".format(rank+1)
+
+        self._filename = filename + self.FILE_EXTENSION
+
+        self.filename_base = filename
 
         check_fitsio()
 
@@ -112,11 +113,16 @@ class FitsOutput(OutputBase):
             key=unreserve_indicator + key
         self._final_metadata[key]= (value, final_metadata_indicator+comment)
 
+    def name_for_sampler_resume_info(self):
+        return self.filename_base + '.sampler_status'
 
     @classmethod
-    def from_options(cls, options):
+    def from_options(cls, options, resume=False):
         #look something up required parameters in the ini file.
         #how this looks will depend on the ini 
+        if resume:
+            raise ValueError("Cannot resume from FITS output")
+            
         filename = options['filename']
         delimiter = options.get('delimiter', '\t')
         rank = options.get('rank', 0)
